if __FILE__ == $0
    $:.unshift '..'
    $:.unshift '../../lib'
    $puppetbase = "../../../../language/trunk"
end

require 'puppet'
require 'puppettest'
require 'test/unit'

# $Id: tc_events.rb 576 2005-08-23 16:09:14Z luke $

class TestEvents < TestPuppet
    def setup
        Puppet[:loglevel] = :debug if __FILE__ == $0
        super
    end

    def test_simplesubscribe
        file = Puppet::Type::PFile.new(
            :name => "/tmp/eventtestingA",
            :create => true
        )
        exec = Puppet::Type::Exec.new(
            :name => "echo true",
            :path => "/usr/bin:/bin",
            :refreshonly => true,
            :subscribe => [[file.class.name, file.name]] 
        )

        @@tmpfiles << "/tmp/eventtestingA"

        comp = Puppet::Type::Component.new(
            :name => "eventtesting"
        )
        comp.push exec
        trans = comp.evaluate
        events = nil
        assert_nothing_raised {
            events = trans.evaluate
        }

        assert_equal(1, events.length)

        assert_equal(1, trans.triggered?(exec, :refresh))
    end

    def test_simplerequire
        file = Puppet::Type::PFile.new(
            :name => "/tmp/eventtestingA",
            :create => true
        )
        exec = Puppet::Type::Exec.new(
            :name => "echo true",
            :path => "/usr/bin:/bin",
            :refreshonly => true,
            :require => [[file.class.name, file.name]] 
        )

        @@tmpfiles << "/tmp/eventtestingA"

        comp = Puppet::Type::Component.new(
            :name => "eventtesting"
        )
        comp.push exec
        trans = comp.evaluate
        events = nil
        assert_nothing_raised {
            events = trans.evaluate
        }

        assert_equal(1, events.length)

        assert_equal(0, trans.triggered?(exec, :refresh))
    end

    def test_ladderrequire
        comps = {}
        objects = {}
        fname = "/tmp/eventtestfuntest"
        [:a, :b].each { |l|
            case l
            when :a
                name = "/tmp/eventtesting%s" % l
                objects[l] = Puppet::Type::PFile.new(
                    :name => name,
                    :create => true
                )
                @@tmpfiles << name
            when :b
                objects[l] = Puppet::Type::Exec.new(
                    :name => "touch %s" % fname,
                    :path => "/usr/bin:/bin",
                    :refreshonly => true
                )
                @@tmpfiles << fname
            end


            comps[l] = Puppet::Type::Component.new(
                :name => "eventtesting%s" % l
            )

            comps[l].push objects[l]
        }

        comps[:b][:subscribe] = [[comps[:a].class.name, comps[:a].name]]

        trans = comps[:a].evaluate
        events = nil
        assert_nothing_raised {
            events = trans.evaluate
        }

        assert(FileTest.exists?(fname))
        #assert_equal(events.length, trans.triggered?(objects[:b], :refresh))
    end
end
